/* "ogl", example of using OpenGL with ESCAPI to create a "funny mirror" effect */

#ifdef _MSC_VER
#include <windows.h> // needed to get GL stuff to work
//#include <time.h>
#endif
#include <SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include "escapi.h"
struct SimpleCapParams capture;
int device;

void render()
{
	int tick = SDL_GetTicks();
	int i, j;

	// Generate a 16x16 grid, and perturb the UV coordinates to 
	// generate the "funny mirrors" effect.
	glBegin(GL_TRIANGLE_STRIP);
	for (j = 0; j < 16; j++)
	{
		for (i = 0; i < 17; i++)
		{
			float u, v;
			u = i * (1 / 16.0f);
			v = j * (1 / 16.0f);
			u += (float)((sin((tick + i*100) * 0.012387) * 0.04) * sin(tick * 0.000514382));
			v += (float)((cos((tick + j*100) * 0.012387) * 0.04) * sin(tick * 0.000714282));
			glTexCoord2f(u, v);
			glVertex2f(i * (640 / 16.0f), j * (480 / 16.0f));

			u = i * (1 / 16.0f);
			v = (j + 1) * (1 / 16.0f);
			u += (float)((sin((tick +       i*100) * 0.012387) * 0.04) * sin(tick * 0.000514382));
			v += (float)((cos((tick + (j + 1)*100) * 0.012387) * 0.04) * sin(tick * 0.000714282));
			glTexCoord2f(u, v);
			glVertex2f(i * (640 / 16.0f), (j + 1) * (480 / 16.0f));
		}
		// create some degenerate surfaces to "turn" the strip after each horizontal span
		glVertex2f(640, (j + 1) * (480 / 16.0f));
		glVertex2f(0, (j + 2) * (480 / 16.0f));
	}
	glEnd();

	// Show what we've got
	SDL_GL_SwapBuffers();
	// Don't hog all CPU time
	SDL_Delay(1);
	
	if (isCaptureDone(device))
	{
		// OpenGL red and blue are swapped, so we'll need to do some converting here..
		for (i = 0; i < 256*256; i++)
			capture.mTargetBuf[i] = (capture.mTargetBuf[i] & 0xff00ff00) |
			                        ((capture.mTargetBuf[i] & 0xff) << 16) |
									((capture.mTargetBuf[i] & 0xff0000) >> 16);

		// Load up the new data
		glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,256,256,0,GL_RGBA,GL_UNSIGNED_BYTE,(GLvoid*)capture.mTargetBuf);
		
		// ..and ask for more
		doCapture(device);
	}
}

// Entry point
int main(int argc, char *argv[])
{
	device = setupESCAPI();
	if (device < 1)
	{
        fprintf(stderr, "Unable to init ESCAPI\n");
        exit(1);
	}

	// Use the second device if more than one found
	// (makes my personal test setup easier =)
	if (device > 1)
		device = 1;
	else
		device = 0;

	// Setup ESCAPI to capture a 256x256 image.
	capture.mWidth = 256;
	capture.mHeight = 256;
	capture.mTargetBuf = new int[256*256];
	initCapture(device,&capture);
	doCapture(device);

	// Initialize SDL's subsystems - in this case, only video.
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) 
	{
		fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
		exit(1);
	}


	// Register SDL_Quit to be called at exit; makes sure things are
	// cleaned up when we quit.
	atexit(SDL_Quit);

    int width = 640;
    int height = 480;
    int bpp = 32;

    SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

	// Add the SDL_FULLSCREEN flag if you want to run fullscreen.
    int flags = SDL_OPENGL;// | SDL_FULLSCREEN;

    if (SDL_SetVideoMode(width, height, bpp, flags) == 0) 
    {
        fprintf( stderr, "Video mode set failed: %s\n", SDL_GetError());
        SDL_Quit();
        exit(0);
    }

	// Set up OpenGL..
    glViewport(0, 0, 640, 480);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

	// Set up the view matrix so that both X and Y coordinates are reversed.
	// OpenGL normally considers 0,0 to be the bottom left corder, so reversing
	// the Y axis makes sense if we want to think the top left to be the origo,
	// and reversing the X axis makes sense since this is supposed to be a "mirror".
	gluOrtho2D(640, 0, 480, 0);
	glColor3f(1,1,1);

	// Create our texture
	glEnable(GL_TEXTURE_2D);
    GLuint texname;
    glGenTextures(1,&texname);
    glBindTexture(GL_TEXTURE_2D,texname);

	// Load up some initial data
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,256,256,0,GL_RGBA,GL_UNSIGNED_BYTE,(GLvoid*)capture.mTargetBuf);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // Linear Filtering
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // Linear Filtering

	// Main loop: loop forever.
	while (1)
	{
		// Render stuff
		render();

		// Poll for events, and handle the ones we care about.
		SDL_Event event;
		while (SDL_PollEvent(&event)) 
		{
			switch (event.type) 
			{
			case SDL_KEYUP:
				switch (event.key.keysym.sym)
				{
				case SDLK_ESCAPE:
					SDL_Quit();
					return 0;
					break;
				}
				break;
			case SDL_QUIT:
				SDL_Quit();
				return(0);
			}
		}
	}
	return 0;
}